/**
 * Copyright (c) 2017-present, Evosphere.
 * All rights reserved.
 */

/**
 * Mock the Header API
 *
 * Fetch returns header in the form of a class, and provide an iterator to loop through the headers.
 * We expect headers to be simple objects for the app to easily change its networking system in need. So we have
 * to mock in fetch directly returning Header objects to handle, so we can have a map in the result.
 *
 * @constructor
 */
export default function HeaderMock(){
  this.map = {};

  this.set = function(key, value){
    this.map[key] = [value];
  };

  this.entries = function(){
    let keys = Object.keys(this.map);
    let length = keys.length;

    return {
      [Symbol.iterator](){
        return {
          i: 0,
          next(){
            if(this.i === length){
              return { done: true }
            }

            // Header entries are in the form of
            // [key, value]
            const returnValue = {
              value: [ keys[this.i], map[keys[this.i]]],
              done: false,
            };

            this.i++;

            return returnValue;
          }
        }
      },
    }
  };
}