'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _reactNative = require('react-native');

/**
 * Handles the network status to avoid doing request if the user is not
 * connected to internet
 * @constructor
 */
function NetworkStatus() {
  var _this = this;

  var checkAtInitialization = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

  var isConnected = false,
      hasBeenChecked = false;

  /**
   * Set the connection info
   * @param isConnectionOk
   */
  this.changeNetworkInfo = function (isConnectionOk) {
    return isConnected = isConnectionOk;
  };

  /**
   * If we didn't checked yet, we assume the user is connected
   * @return boolean
   */
  this.isConnected = function () {
    return hasBeenChecked === false ? true : isConnected;
  };

  /**
   * Check the connection status
   * @returns {Promise.<void>}
   */
  this.check = async function () {
    var result = await _reactNative.NetInfo.isConnected.fetch();
    _this.changeNetworkInfo(result);
    hasBeenChecked = true;
  };

  _reactNative.NetInfo.isConnected.addEventListener('change', this.changeNetworkInfo);
  checkAtInitialization && this.check();
}
/**
 * Copyright (c) 2017-present, Evosphere.
 * All rights reserved.
 */

exports.default = {
  getInstance: function getInstance() {
    if (!this.instance) {
      this.instance = new NetworkStatus();
    }

    return this.instance;
  }
};