"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; /**
                                                                                                                                                                                                                                                                               * Copyright (c) 2017-present, Evosphere.
                                                                                                                                                                                                                                                                               * All rights reserved.
                                                                                                                                                                                                                                                                               */

var _invariant = require("invariant");

var _invariant2 = _interopRequireDefault(_invariant);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Provides an interface around a SecurityToken system.
 * We leave the implementation to the user so he can decide from which source to fetch data. In production it will
 * usually be the store.
 *
 * @constructor
 */
function SecurityToken() {
  this.implementation = null;
}

SecurityToken.prototype.assertIsImplemented = function () {
  (0, _invariant2.default)(_typeof(this.implementation) === "object" && this.implementation !== null && typeof this.implementation.setToken === "function" && typeof this.implementation.getToken === "function", "You must provide an implementation to SecurityToken.");
};

SecurityToken.prototype.setImplementation = function (impl) {
  this.implementation = impl;
};

SecurityToken.prototype.setToken = function (token) {
  this.assertIsImplemented();
  this.implementation.setToken(token);
};

SecurityToken.prototype.getToken = function () {
  this.assertIsImplemented();
  return this.implementation.getToken();
};

exports.default = {
  /**
   * @returns {container}
   */
  createFromContainer: function createFromContainer(container) {
    return new SecurityToken(container.get("store"));
  }
};