'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _redux = require('redux');

var _reactNative = require('react-native');

var _reduxBatch = require('../util/reduxBatch');

var _reduxPersist = require('redux-persist');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function StoreManager() {
  this._store = null;
  this._isConfigured = false;
}
/**
 * Copyright (c) 2017-present, Evosphere.
 * All rights reserved.
 */

var defaultInitConfiguration = {
  debounce: 500
};

/**
 * Creates the store and rehydrates it
 *
 * @param config
 * @returns {Promise}
 */
StoreManager.prototype.init = function (config) {
  var _this = this;

  config = Object.assign({}, defaultInitConfiguration, config);

  this._store = (0, _redux.createStore)((0, _redux.combineReducers)(config.reducers), (0, _redux.compose)(_reduxBatch.reduxBatch, (0, _reduxPersist.autoRehydrate)()));

  return new Promise(function (accept) {
    (0, _reduxPersist.persistStore)(_this._store, {
      debounce: config.debounce,
      storage: _reactNative.AsyncStorage
    }, function () {
      _this._isConfigured = true;
      accept(_this._store);
    });
  });
};

/**
 * Throws an error if the store is not configured
 */
StoreManager.prototype.assertStoreIsConfigured = function () {
  (0, _invariant2.default)(this.isConfigured() === true, "You cannot access any property of the store as it is not configured yet. " + "Please call the init function to configure the store.");
};

/**
 * Return the state
 * @returns {S}
 */
StoreManager.prototype.getState = function () {
  this.assertStoreIsConfigured();
  return this._store.getState();
};

/**
 *
 * @returns {*<S>}
 */
StoreManager.prototype.dispatch = function (action) {
  this.assertStoreIsConfigured();
  return this._store.dispatch(action);
};

/**
 * Returns true if the store is configured
 * @returns {boolean}
 */
StoreManager.prototype.isConfigured = function () {
  return this._isConfigured;
};

/**
 * Return the store
 * @returns {Store<S>|*|null}
 */
StoreManager.prototype.getStore = function () {
  return this._store;
};

exports.default = {
  create: function create() {
    return new StoreManager();
  }
};