'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.withTimeout = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };
/**
 * Copyright (c) 2017-present, Evosphere.
 * All rights reserved.
 */


var _objectMap = require('evojs/lib/objectMap');

var _objectMap2 = _interopRequireDefault(_objectMap);

var _isArray = require('evojs/lib/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

var _isFixedNumber = require('evojs/lib/isFixedNumber');

var _isFixedNumber2 = _interopRequireDefault(_isFixedNumber);

var _encodeObjectToURI = require('evojs/lib/encodeObjectToURI');

var _encodeObjectToURI2 = _interopRequireDefault(_encodeObjectToURI);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var DEFAULT_TIMEOUT = 5000;

/**
 * Return a timeout if any, or by default return 5000
 * @param obj
 * @returns {number}
 */
function getTimeoutDelay(obj) {
  if ((0, _isFixedNumber2.default)(obj.timeout)) {
    return obj.timeout;
  }

  return DEFAULT_TIMEOUT;
}

/**
 * Map the headers into a map of headers with scalar values
 * @param headers
 */
function mapHeaders(headers) {
  return (0, _objectMap2.default)(headers, function (value) {
    if ((0, _isArray2.default)(value) && value.length === 1) {
      return value[0];
    }

    return value;
  });
}

var TimeoutException = {
  name: "TimeoutException"
};

/**
 * make sure the promise rejects once the delay has expired
 * @param ms
 * @param promise
 * @returns {Promise}
 */
var withTimeout = exports.withTimeout = function withTimeout(ms, promise) {
  return new Promise(function (resolve, reject) {
    setTimeout(function () {
      reject(TimeoutException);
    }, ms);

    promise.then(resolve, reject);
  });
};

function urlAlreadyHasParameters(url) {
  return url.indexOf("?") > -1;
}

function transformURLWithQuery(url, parameters) {
  if (urlAlreadyHasParameters(url)) {
    return url + "&" + parameters;
  }

  return url + "?" + parameters;
}
/**
 * Creates an adapter around the Fetch API
 * @param fetchFn
 */
var createAdapter = function createAdapter(fetchFn) {
  return function () {
    var defaultHeaders = {
      'Content-Type': 'application/x-www-form-urlencoded'
    };

    return {
      sendRequest: function sendRequest(request) {
        var config = {
          method: request.method,
          headers: defaultHeaders
        };

        if (request.headers) {
          config.headers = request.headers;
        }

        // We send POST parameters as x-www-form-urlencoded
        if (request.request && _typeof(request.request) === "object") {
          if (!request.request._parts) {
            config.body = (0, _encodeObjectToURI2.default)(request.request);
          } else {
            // $FlowFixMe
            config.body = request.request;
          }
        }

        // GET parameters must be passed in the URL
        var url = request.url;
        if (request.query) {
          url = transformURLWithQuery(request.url, (0, _encodeObjectToURI2.default)(request.query));
        }

        return withTimeout(getTimeoutDelay(request), fetchFn(url, config)).then(function (response) {
          return response.json().then(function (data) {
            return {
              status: "done",
              response: {
                statusCode: response.status,
                headers: mapHeaders(response.headers.map),
                data: data
              }
            };
          }).catch(function (e) {
            if (e.name === "SyntaxError") {
              return {
                status: "done",
                response: {
                  statusCode: response.status,
                  headers: mapHeaders(response.headers.map),
                  data: null
                }
              };
            }
          });
        }).catch(function (error) {
          // See withTimeout
          if (error === TimeoutException) {
            return {
              status: "timeout",
              response: null
            };
          }

          return {
            status: "network-error",
            response: null
          };
        });
      }
    };
  };
};

exports.default = createAdapter;