"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = interpretResult;


/**
 * Return true if the value is between the min & max
 *
 * @param value
 * @param min
 * @param max
 * @returns {boolean}
 */
function isBetween(value, min, max) {
  return value >= min && value <= max;
}

/**
 * Gives an interpretation of a request result
 * This allows to have nice string to understand what happened, like "success" or "client-error"
 *
 * @param result
 * @returns {ResultInterpretationType}
 */

/**
 * Copyright (c) 2017-present, Evosphere.
 * All rights reserved.
 */

function interpretResult(result) {
  if (!result) {
    return "unknown";
  }

  if (result.response) {
    if (result.status === "done" && isBetween(result.response.statusCode, 200, 299)) {
      return "success";
    }

    if (result.status === "done" && isBetween(result.response.statusCode, 400, 499)) {
      return "client-error";
    }

    if (result.status === "done" && isBetween(result.response.statusCode, 500, 599)) {
      return "server-error";
    }
  }

  if (result.status === "timeout") {
    return "timeout";
  }

  if (result.status === "network-error") {
    return "network-error";
  }

  return "unknown";
};