"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.reduxBatch = reduxBatch;
/**
 * Copyright (c) 2017-present, Evosphere.
 * All rights reserved.
 */

function isArray(item) {
  return Array.isArray(item);
}

function isFunction(item) {
  return typeof item === "function";
}

function reduxBatch(next) {
  var nextListeners = [];
  var currentListeners = currentListeners;

  function ensureCanMutateNextListeners() {
    if (nextListeners === currentListeners) {
      nextListeners = nextListeners.slice();
    }
  }

  function subscribe(listener) {
    if (!isFunction(listener)) throw new Error("Invalid listener, expected a function");

    var isSubscribed = true;

    ensureCanMutateNextListeners();
    nextListeners.push(listener);

    return function unsubscribe() {
      if (!isSubscribed) return;

      ensureCanMutateNextListeners();
      nextListeners.splice(nextListeners.indexOf(listener), 1);
    };
  }

  function notifyListeners() {
    var listeners = nextListeners;

    for (var t = 0; t < listeners.length; ++t) {
      currentListeners = listeners;
      listeners[t]();
    }
  }

  return function () {
    var store = next.apply(undefined, arguments);

    function dispatchRecurse(action) {
      return isArray(action) ? action.map(function (subAction) {
        return dispatchRecurse(subAction);
      }) : store.dispatch(action);
    }

    function dispatch(action) {
      var result = dispatchRecurse(action);
      notifyListeners();
      return result;
    }

    return Object.assign({}, store, {
      dispatch: dispatch, subscribe: subscribe
    });
  };
}