/**
 * Copyright (c) 2017-present, Evosphere.
 * All rights reserved.
 */

import invariant from 'invariant';

import StoreManager from './services/StoreManager';
import NetworkStatus from './services/NetworkStatus';
import SecurityToken from './services/SecurityToken';
import RequestSender from './services/requestSender/RequestSender';
import Faussaire from 'faussaire';

import faussaireAdapter from './services/requestSender/adapter/faussaireAdapter';
import fetchAdapter from './services/requestSender/adapter/fetchAdapter';

export default function ShaaraReactNativeTask(){
  return {
    getServices(container, parameters, env){
      let faussaire = null;
      const networkStatus = NetworkStatus.getInstance();
      const requestSenderImplementation = (() => {
        if(env === "test"){
          faussaire = Faussaire.create();
          return faussaireAdapter(faussaire);
        }

        // If there's an user-defined fetch function, we can use it
        if(parameters.fetch){
          return fetchAdapter(parameters.fetch);
        }

        if(typeof fetch !== "function"){
          invariant(
            false,
            "fetch is not defined. Cannot initialize the request sender."
          );
        }

        return fetchAdapter(fetch);
      })();

      return {
        "store": StoreManager.create(),
        "network.status": networkStatus,
        "security.token": SecurityToken.createFromContainer(container),
        "request.sender": RequestSender.create(requestSenderImplementation, networkStatus.isConnected.bind(networkStatus)),
        ...(faussaire !== null ? { "faussaire" : faussaire } : {})
      };
    },
    async configureKernel(config, kernel){
      if(typeof config.getStoreConfiguration === "function"){
        await kernel.get("store").init(config.getStoreConfiguration());
      }

      if(typeof config.getTokenImplementation === "function"){
        kernel.get("security.token").setImplementation(config.getTokenImplementation(kernel._services));
      }
    }
  }
}