// @flow
/**
 * Copyright (c) 2017-present, Evosphere.
 * All rights reserved.
 */

import { NetInfo } from 'react-native';

/**
 * Handles the network status to avoid doing request if the user is not
 * connected to internet
 * @constructor
 */
function NetworkStatus(checkAtInitialization : boolean = true){
  let
    isConnected = false,
    hasBeenChecked = false
  ;

  /**
   * Set the connection info
   * @param isConnectionOk
   */
  this.changeNetworkInfo = (isConnectionOk) => isConnected = isConnectionOk;

  /**
   * If we didn't checked yet, we assume the user is connected
   * @return boolean
   */
  this.isConnected = () : boolean => hasBeenChecked === false ? true : isConnected;

  /**
   * Check the connection status
   * @returns {Promise.<void>}
   */
  this.check = async () => {
    let result = await NetInfo.isConnected.fetch();
    this.changeNetworkInfo(result);
    hasBeenChecked = true;
  };

  NetInfo.isConnected.addEventListener('change', this.changeNetworkInfo);
  checkAtInitialization && this.check();
}

export default {
  getInstance(){
    if(!this.instance){
      this.instance = new NetworkStatus();
    }

    return this.instance;
  }
}