/**
 * Copyright (c) 2017-present, Evosphere.
 * All rights reserved.
 */

import invariant from 'invariant';

export type SecurityTokenType<$TokenType : mixed> = {
  setToken: (token: $TokenType) => void,
  getToken: () => $TokenType,
  getAppUserId: () => string,
};

/**
 * Provides an interface around a SecurityToken system.
 * We leave the implementation to the user so he can decide from which source to fetch data. In production it will
 * usually be the store.
 *
 * @constructor
 */
function SecurityToken(){
  this.implementation = null;
}

SecurityToken.prototype.assertIsImplemented = function(){
  invariant(
    typeof this.implementation === "object"
    && this.implementation !== null
    && typeof this.implementation.setToken === "function"
    && typeof this.implementation.getToken === "function",
    "You must provide an implementation to SecurityToken."
  )
};

SecurityToken.prototype.setImplementation = function(impl){
  this.implementation = impl;
};

SecurityToken.prototype.setToken = function(token){
  this.assertIsImplemented();
  this.implementation.setToken(token);
};

SecurityToken.prototype.getToken = function(){
  this.assertIsImplemented();
  return this.implementation.getToken();
};

export default {
  /**
   * @returns {container}
   */
  createFromContainer(container) : SecurityTokenType {
    return new SecurityToken(container.get("store"));
  }
}