// @flow
/**
 * Copyright (c) 2017-present, Evosphere.
 * All rights reserved.
 */

import type {
  ResultType,
} from './definitionTypes';

class HttpRequestException extends Error {
  constructor(message: string){
    super(message);
    this.name = "HttpRequestException";
  }
}

/**
 * Ensure the result is having the correct format
 * @param result
 */
export default function checkResultFormat(result : ?ResultType){
  if(typeof result !== "object" || result === null){
    throw new HttpRequestException("The result must be an object");
  }

  if(typeof result.status !== "string" || (result.status === "done" && (typeof result.response !== "object" || result.response === null))){
    throw new HttpRequestException(
      "The result object is invalid. If the status is set to \"done\", you must provide a response object."
    );
  }

  if(typeof result.response !== "object" && result.response !== null){
    if(typeof result.response.statusCode !== "number"){
      throw new HttpRequestException("The result must have a status code that is a number.");
    }
  }
};