// @flow
/**
 * Copyright (c) 2017-present, Evosphere.
 * All rights reserved.
 */

export type HttpMethod = "POST" | "GET" | "PATCH" | "OPTIONS" | "HEAD" | "DELETE";

/**
 * Correspond to a request configuration. This is the object
 * sent to HttpRequestSenderFactory.send method.
 */
export type RequestConfiguration = {
  url: string,
  method: HttpMethod,
  query?: ?Object,
  request?: ?Object,
  headers?: ?Object,
  timeout?: ?Number,

  transformer?: (request : ?Object) => any,
  payload?: any,
};

export type ResponseType = {
  data: mixed,
  statusCode: number,
  headers: ?Object,
};

/**
 * Interface corresponding to an HTTP operation result.
 * Note that an operation isn't the request itself. Because such operation might not be triggered
 * (when there's no connection for example) or canceled. In this case it's wrong to talk about an HTTP Response.
 * So the Result holds the HTTP Response if any, and the operation status (either "done" or "timeout").
 */
export type ResultType = {
  response: ?ResponseType,
  status: "done" | "timeout" | "network-error",
};

export type ResultInterpretationType = "success" | "client-error" | "server-error" | "network-error" | "timeout" | "unknown";

export type NetworkInfoType = {
  isConnected: () => boolean,
};